<!doctype html>
<html lang="en">

{% raw %}
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    {% block bootstrap_css %}
    {{ bootstrap.load_css() }}
    {% endblock %}

    {% block styles %}
    {% endblock %}
</head>
{% endraw %}

<body>
    <nav class="navbar navbar-expand-md navbar-dark fixed-top bg-dark">
        <a class="navbar-brand" href="#">{{cookiecutter.name}}</a>

        <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarCollapse">
            <span class="navbar-toggler-icon"></span>
        </button>

        <div class="collapse navbar-collapse" id="navbarCollapse">
            <ul class="navbar-nav mr-auto">
                <li class="nav-item active">
                    <a class="nav-link" href="#">Home <span class="sr-only">(current)</span></a>
                </li>
                <li class="nav-item">
                    <a class="nav-link" href="#">Link</a>
                </li>
            </ul>
        </div>
    </nav>

    {% raw %}
    <main role="main" class="container">
        {% block content %}{% endblock %}
    </main>
    {% endraw %}


    {% raw %}
    {% block bootstrap_js %}
    {{ bootstrap.load_js() }}
    {% endblock %}

    {% block scripts %}
    {% endblock %}
    {% endraw %}
</body>

</html>
